// basic functions

gmexport double ds_bin_create(double bits, double size_x, double size_y, double size_z) {
	ds_bin *bin;
	bin = create_bin();
	if(bin==NULL) {
		return 0;
	}
	bin->bits = (unsigned char) bits;
	if(bin->bits==0) {
		destroy_bin(bin);
		return -1;
	}
	bin->x = (unsigned long) size_x;
	bin->y = (unsigned long) size_y;
	bin->z = (unsigned long) size_z;
	if(32%bin->bits!=0) {
		destroy_bin(bin);
		return -1;
	}
	bin->entriesperword = 32/bin->bits;
	bin->entries = bin->x*bin->y*bin->z;
	bin->count = bin->entries/bin->entriesperword+((bin->entries%bin->entriesperword==0)?(0):(1));
	bin->size = bin->count*sizeof(uint32_t);
	bin->data = (uint32_t*) malloc(bin->size);
	if(bin->data==NULL) {
		destroy_bin(bin);
		return 0;
	}
	memset(bin->data,0,bin->size);
	bin->id = ++id_bins;
	return bin->id;
}

gmexport double ds_bin_destroy(double id) {
	ds_bin *bin;
	if((bin=search_bin((unsigned long) id))==NULL) return -1;
	if(selectedbin==bin) {
		selectedbin = NULL;
	}
	free(bin->data);
	destroy_bin(bin);
	return 1;
}

gmexport double ds_bin_clear(double id, double val) {
	unsigned long i;
	uint32_t value,fill;
	ds_bin *bin;
	if((bin=search_bin((unsigned long) id))==NULL) return -1;
	value = (uint32_t) val;
	fill = 0;
	for(i=0;i<bin->entriesperword;++i) {
		fill |= value<<(i*bin->bits);
	}
	for(i=0;i<bin->count;++i) {
		bin->data[i] = fill;
	}
	return 1;
}

gmexport double ds_bin_select(double id) {
	ds_bin *bin;
	if((bin=search_bin((unsigned long) id))==NULL) return -1;
	selectedbin = bin;
	return 1;
}

gmexport double ds_bin_get_x() { return return_x; }
gmexport double ds_bin_get_y() { return return_y; }
gmexport double ds_bin_get_z() { return return_z; }

char *returnstring;

gmexport char* ds_bin_serialize(double id) {
	unsigned long i;
	ds_bin *bin;
	if((bin=search_bin((unsigned long) id))==NULL) return "";
	free(returnstring);
	returnstring = (char*) malloc(bin->size*2+1);
	if(returnstring==NULL) return "";
	for(i=0;i<bin->size;++i) {
		unsigned char a = ((char*)(bin->data))[i];
		returnstring[2*i]   = ((a>>4)>9)? 'a'+(a>>4)-10 : '0'+(a>>4);
		returnstring[2*i+1] = ((a&15)>9)? 'a'+(a&15)-10 : '0'+(a&15);
	}
	returnstring[bin->size*2] = '\0';
	return returnstring;
}

gmexport double ds_bin_unserialize(double id, char* data) {
	unsigned long i;
	ds_bin *bin;
	if((bin=search_bin((unsigned long) id))==NULL) return -1;
	if(strlen(data)!=bin->size*2) return -1;
	for(i=0;i<bin->size;++i) {
		unsigned char a = data[2*i];
		unsigned char b = data[2*i+1];
		((char*)(bin->data))[i] =
			((('0'<=a && a<='9')? a-'0' : (('a'<=a && a<='f')? 10+a-'a' : (('A'<=a && a<='F')? 10+a-'A' : 0)))<<4) |
			(('0'<=b && b<='9')? b-'0' : (('a'<=b && b<='f')? 10+b-'a' : (('A'<=b && b<='F')? 10+b-'A' : 0)));
	}
	return 1;
}

